MAPBOX_ACCESS_TOKEN = 'pk.eyJ1IjoibWFwYm94IiwiYSI6ImNpejY4NXVycTA2emYycXBndHRqcmZ3N3gifQ.rJcFIG214AriISLbB6B5aw';
MAX_OPACITY = 1;
FIRST_IMAGE_LAYER_INDEX = 2;

// create wkt polygon
const wktFormat = new ol.format.WKT();
const polygonFeature = wktFormat.readFeature(polygon, {
  dataProjection: 'EPSG:4326',
	featureProjection: 'EPSG:3857'
});

// Create layers
const mapLayer = new ol.layer.Tile({
  name: 'mapbox',
  source: new ol.source.XYZ({
    url:'https://api.tiles.mapbox.com/v4/mapbox.light/{z}/{x}/{y}.png?access_token='+MAPBOX_ACCESS_TOKEN
  })
});

const polygonLayer = new ol.layer.Vector({
  name: 'polygon',
  source: new ol.source.Vector({
    features: [polygonFeature]
  })
});

const layers = new Map();

// create map
const map = new ol.Map({
  target: 'minimap',
  layers: [mapLayer, polygonLayer],
  view: new ol.View({
    center: ol.proj.fromLonLat([-99.19, 19.61]),
    zoom: 4,
    minZoom:4
  })
});

// fit (zoom) to polygon
map.getView().fit(polygonFeature.getGeometry());

// show first layer date
let currentDate = labels[0];
addImageLayer({
  layerDate: currentDate
});

// main function to change layers
// data = { layerDate: to change all layers by date }
// data = { layerName: to add single layer }
function addImageLayer(data) {
  if (data.layerDate) {
    currentDate = data.layerDate;
  }

  $.ajax({
    type: 'POST',
    dataType: "json",
    data: {'purchase_id': purchase_id, 'report': zone, 'date': currentDate},
    url: '/reports/tiles/tileinfo',
    success: response => {
      changeLayers(data.layerName);
      setOpacity($('#opacitySlide').val());
      map.getView().setMinZoom(response.zoom.min);
      map.getView().setMaxZoom(response.zoom.max);
    }
  });
}

// remove old layers and add layers with new date
function changeLayers(especific_layerName) {
  const layersToChance = especific_layerName ? especific_layerName.split() : getSelectedLayersName();

  removeLayer(especific_layerName);

  layersToChance.forEach(layerName => {
    const xyz = new ol.layer.Tile({
      name: layerName,
      source: new ol.source.XYZ({
        projection: 'EPSG:3857',
        url: '/reports/tiles/'+purchase_id+'/'+zone+'/'+layerName+'/'+currentDate+'/{z}/{x}/{-y}.png',
        tilePixelRatio: 2
      })
    });

    if (layerName === 'TCI') {
      map.getLayers().insertAt(FIRST_IMAGE_LAYER_INDEX, xyz);
    } else {
      map.addLayer(xyz);
    }

    layers.set(layerName, xyz);
  });
}

// remove all if there isn't layername
function removeLayer(layerName) {
  if (!layerName) {
    layers.forEach(layer => { map.removeLayer(layer); });
    layers.clear();
    return;
  }

  map.removeLayer(layers.get(layerName));
  layers.delete(layerName);
}

// set opacity to layers
function setOpacity(value) {
  layers.forEach(layer => {
    if(layer.get('name') != 'TCI') {
      layer.setOpacity(value)
    }
  })
}


// *********** Irving ******************
//  Funciones de comparacion

//vigila cuando se cierra el menu de comparar y cuando se cierra desactiva la comparacion (si la clase de compare incluye menu-open está abierto)
var compareMenu = document.getElementById('compare');
var observer = new MutationObserver( function(mutations) {
  mutations.forEach( function(mutation) {
    if (mutation.attributeName == 'class') {
      if(compareMenu.className.indexOf("open")===-1) {  //se cerró el menu de comparacion
        exitCompare();
      }
    }
  });
});
observer.observe(compareMenu, { attributes : true, attributeOldValue: true, attributeFilter : ['class'], subtree: true});

//crea el control swipe, que sirve para comparar
var ctrl = new ol.control.Swipe();

//funcion que inicia la comparacion
//recibe la lista de fechas y por default compara la primera y la ultima
function compare(labels) {
  var swipeControl = false;

  map.getControls().forEach(function (control) {  //checa si ya se añadio el control de comparacion (swipe), si no lo tiene, lo agrega
    if(control instanceof ol.control.Swipe){ //si ya tiene el control agregado
      swipeControl =true;
    }
  });

  if(swipeControl==true) {  //si ya tiene el comparador no hace nada, ya esta en modo comparacion
    //map.removeControl(swipeControlObj);
  } else {  //si no tiene el comparador lo agrega
    //removeImageLayers();
    map.addControl(ctrl);
    ctrl.set('position', 0.5);
    //console.log(map.controls)
    var labelDateLeft = labels[0]                 //por default compara la primera y ultima fecha
    var labelDateRight = labels[labels.length-1]

    //pone las fechas por default en los selectores de fechas
    document.getElementById("selectLeftCompare").value = labelDateLeft;
    document.getElementById("selectRightCompare").value = labelDateRight;

     //agrega las capas izquierda/derecha
    selectLeft(labelDateLeft);
    selectRight(labelDateRight);

    //muestra los selectores de fechas y oculta la linea de tiempo
    document.getElementById("selectLeft").style.display = "block";
    document.getElementById("selectRight").style.display = "block";
    document.getElementById("timeLine").style.display = "none";
  }
}

//sale del modo comparacion
function exitCompare() {
  map.getControls().forEach(function (control) {  //checa si ya se añadio el control de comparacion (swipe)
    if(control instanceof ol.control.Swipe){ //ya tiene el control agregado lo quita
      map.removeControl(control);
    }
  });

  //quita las capas de comparacion izquierda/derecha
  var imageLayers = map.getLayers().getArray().slice();
  imageLayers.forEach(layer => {
    if(layer.get('name') === 'left' || layer.get('name') === 'right') {
      map.removeLayer(layer)
    }
  })

  //oculta los selectores de fechas y muestra la linea de tiempo
  document.getElementById("selectLeft").style.display = "none";
  document.getElementById("selectRight").style.display = "none";
  document.getElementById("timeLine").style.display = "block";
}


//selecciona el lado izquierdo para comparar
function selectLeft(date) {
  var labelDateLeft = date;
  var layer_type = 'TCI';
  const left = new ol.layer.Tile({
    name: 'left',
    opacity: 1,
    source: new ol.source.XYZ({
      projection: 'EPSG:3857',
      url: '/reports/tiles/'+purchase_id+'/'+zone+'/'+layer_type+'/'+labelDateLeft+'/{z}/{x}/{-y}.png',
      tilePixelRatio: 2,
      minZoom: 10,
      maxZoom: 15
    })
  });
  map.addLayer(left);
  ctrl.addLayer(left);
}

//selecciona el lado derecho para comparar
function selectRight(date){
    var labelDateRight = date;
    var layer_type = 'TCI';
    const right = new ol.layer.Tile({
      name: 'right',
      opacity: 1,
      source: new ol.source.XYZ({
        projection: 'EPSG:3857',
        url: '/reports/tiles/'+purchase_id+'/'+zone+'/'+layer_type+'/'+labelDateRight+'/{z}/{x}/{-y}.png',
        tilePixelRatio: 2,
        minZoom: 10,
        maxZoom: 15
      })
    });
    map.addLayer(right);
    ctrl.addLayer(right, true);
}

//selecciona comparacion vertical u horizontal
function setCompareDirection(direction){
  ctrl.set('orientation',direction)

  //mueve los selectores de fechas
  if(direction === 'horizontal'){
    document.getElementById("selectLeft").style.bottom = "70%";
    document.getElementById("selectLeft").style.left = "15%";
    document.getElementById("selectRight").style.left = "15%";
  }
  else{
    document.getElementById("selectLeft").style.bottom = "50px";
    document.getElementById("selectLeft").style.left = "30%";
    document.getElementById("selectRight").style.left = "75%";
  }
}
//  Fin funciones de comparacion
//******************************
